/***** ARCHIVO vector.cpp  *****/
#include <iostream>
#include "vector.h"

using namespace std;



///////////////////////////////////////////////////////////////////////////
//
// Constructores y destructor
//
///////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////// 
// Funcion: Vector  (constructor sin argumentos)
//         Observacion: funcion polimorfica                                       
// Objetivo:
//      Construir un objeto de la clase vector con 5 elementos
// Descripcion: 
//       Se crea un objeto de la clase vector 
//       y se le asignan a las componentes el valor 0   
// Entrada:                                                            
//   - No tiene argumentos                                     
// Devuelve: no devuelve nada                 
// Dependencias:  set_numero_elementos, rellenarVector                          
///////////////////////////////////////////////////////////////////////////
Vector::Vector()
{
  // El número de elementos por defecto del vector es 5
  set_numero_elementos(5);

  // Se reserva memoria al vector
  _v = new int [numero_elementos()];

  //Inicializa las componentes del vector con el valor 0
  rellenar_Vector(0); 
}


/////////////////////////////////////////////////////////////////////////// 
// Funcion: Vector  (constructor con un argumento)
//         Observacion: funcion polimorfica                                       
// Objetivo:
//      Construir un objeto de la clase vector 
// Descripcion: 
//       Se crea un objeto de la clase vector con "n" componentes
//       y se le asignan a las componentes el valor 0   
// Entrada:                                                            
//   - n: número de componentes del vector                                    
// Devuelve: no devuelve nada                 
// Dependencias: set_numero_elementos, rellenarVector                    
///////////////////////////////////////////////////////////////////////////

Vector::Vector(int n)
{
  // Se indica el número de componentes del vector
  set_numero_elementos(n);

  // Se reserva memoria
  _v = new int [numero_elementos()];

  //Inicializa las componentes del vector con el valor 0
  rellenar_Vector(0);
}


/////////////////////////////////////////////////////////////////////////// 
// Funcion: ~Vector                                      
// Objetivo:
//      Destruir un objeto de la clase vector 
// Descripcion: 
//       Destruye un objeto de la clase vector y libera su memoria  
// Entrada:                                                            
//   - No tiene argumentos                                     
// Devuelve: no devuelve nada                 
// Dependencias: delete                   
///////////////////////////////////////////////////////////////////////////


// El destructor libera la memoria reservada al vector
Vector::~Vector()
{
  delete [] _v;
}


///////////////////////////////////////////////////////////////////////////
//
// Funciones de modificacion
//
///////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////// 
// Funcion: set_numero_elementos                       
// Objetivo:
//      Modificar la longitud del vector 
// Descripcion: 
//      Modifica la longitud del vector con el argumento indicado
// Entrada:        
//    - n: nueva longitud del vector
// Devuelve: longitud del vector        
// Dependencias: no tiene            
///////////////////////////////////////////////////////////////////////////

void Vector::set_numero_elementos(int n)
{
  _n = n;
}


/////////////////////////////////////////////////////////////////////////// 
// Funcion: set_elemento                         
// Objetivo:
//      Asigna un valor a una componente del vector
// Descripcion: 
//      Asigna un valor a la componente indicada por el argumento - 1
// Entrada:        
//   - valor: dato que se va a asignar a la componente                                               
//   - posicion: indica una de las componentes del vector  
// Devuelve: no devuelve nada         
// Dependencias: no tiene                 
///////////////////////////////////////////////////////////////////////////

void Vector::set_elemento(int valor, int posicion)
{
  _v[posicion-1] = valor;
}



/////////////////////////////////////////////////////////////////////////// 
// Funcion: rellenar_Vector    
//          Observacion: funcion polimorfica                     
// Objetivo:
//      Asigna valores a las componentes de un vector desde el teclado
// Descripcion: 
//      Solicita por pantalla los valores de las componentes del vector
//      para ser introducidos desde el teclado (o flujo de entrada estandar)
//
//      El vector comienza su indexacion a partir de 1
// Entrada:        
//    - no tiene  
// Devuelve: no devuelve nada         
// Dependencias: set_elemento            
///////////////////////////////////////////////////////////////////////////

void Vector::rellenar_Vector()
{
  int valor;

  for(int i = 1; i <= numero_elementos(); i++)
  {
    cout << "Elemento " << i << ": ";
    cin >> valor;
    set_elemento(valor, i);
  }
}



/////////////////////////////////////////////////////////////////////////// 
// Funcion: rellenar_Vector   
//          Observacion: funcion polimorfica                      
// Objetivo:
//      Asigna un mismo valor a todas las componentes de un vector 
// Descripcion: 
//      El valor pasado como argumento se asigna a todas las componentes del vector
//     
//      El vector comienza su indexacion a partir de 1
// Entrada:        
//    - valor: dato que se va a asignar a las componentes del vector
// Devuelve: no devuelve nada         
// Dependencias: set_elemento            
///////////////////////////////////////////////////////////////////////////

void Vector::rellenar_Vector(int valor)
{
  for(int i = 1; i <= numero_elementos(); i++)
  {
    set_elemento(valor, i);
  }
}


///////////////////////////////////////////////////////////////////////////
//
// Funciones de consulta
//
///////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////// 
// Funcion: get_elemento                                      
// Objetivo:
//      Devolver el valor de una componente del vector
// Descripcion: 
//       Devuelve el valor de la componente indicada por el argumento - 1
// Entrada:                                                            
//   - posicion: indica una de las componentes del vector                               
// Devuelve: el valor de la componente indicada                
// Dependencias: no tiene                 
///////////////////////////////////////////////////////////////////////////

int Vector::get_elemento(int posicion)
{
  return _v[posicion-1];
}


/////////////////////////////////////////////////////////////////////////// 
// Funcion: ver_Vector                         
// Objetivo:
//      Mostrar los valores de un vector
// Descripcion: 
//      Escrbe los valores de las componentes de un vector
//      El vector comienza su indexacion a partir de 1
// Entrada:        
//   - valor: dato que se va a asignar a la componente                                               
//   - posicion: indica una de las componentes del vector  
// Devuelve: no devuelve nada         
// Dependencias: no tiene                 
///////////////////////////////////////////////////////////////////////////
void Vector::ver_Vector()
{
  cout << "(";
  for(int i = 1; i < numero_elementos(); i++)
    cout << get_elemento(i) << ", ";

  cout << get_elemento( numero_elementos()) << ")\n";
}



/////////////////////////////////////////////////////////////////////////// 
// Funcion: numero_elementos                       
// Objetivo:
//      Indica la longitud del vector 
// Descripcion: 
//      Devolver la longitud del vector
// Entrada:        
//    - no tiene
// Devuelve: longitud del vector        
// Dependencias: no tiene            
///////////////////////////////////////////////////////////////////////////

int Vector::numero_elementos()
{
  return _n;
}


